<?php
/* Prohibit direct script loading */
defined('ABSPATH') || die('No direct script access allowed!');

use WPLG\Helper;
require_once ABSPATH . '/wp-admin/includes/image.php';
/**
 * Class WpLoadGalleryFront
 * This class that holds most of the front-end functionality for WP Media Folder Gallery
 */
class WpLoadGalleryFront
{
    /**
     * WpLoadGalleryFront constructor.
     */
    public function __construct()
    {
        if (!is_admin()) {
            add_shortcode('wplg_gallery', array($this, 'galleryShortcode'));
        }
        add_action('wp_ajax_wplg_loadmore_gallery', array($this, 'loadmoreGallery'));
        add_action('wp_ajax_nopriv_wplg_loadmore_gallery', array($this, 'loadmoreGallery'));
    }

    /**
     * Run shortcode gallery
     *
     * @param array $attr Params of gallery
     *
     * @return string
     */
    public function galleryShortcode($attr)
    {
        /* Get all params */
        if (empty($attr['gallery_id'])) {
            return '<p style="color: #f00">' . esc_html__('Gallery not found', 'wp-load-gallery') . '</p>';
        }

        $term_exist = get_term((int) $attr['gallery_id'], WPLG_TAXONOMY);
        if (empty($term_exist)) {
            return '<h3>' . __('Gallery not found!', 'wp-load-gallery') . '</h3>';
        }

        static $instance = 0;
        $instance++;
        $theme = get_term_meta((int) $attr['gallery_id'], 'wplg_theme', true);
        $options = get_term_meta((int) $attr['gallery_id'], 'wplg_options', true);
        $params = $this->getSettingsFront($attr['gallery_id'], $attr, $options);
        foreach ($params as $attr_key => $attr_value) {
            ${$attr_key} = $attr_value;
        }

        $theme = Helper::getTheme($theme);
        // ============================
        $is_max_width = (!empty($params['gallery_full_width']) || !isset($params['gallery_full_width'])) ? true : false;
        $gallery_width = (isset($params['gallery_width'])) ? $params['gallery_width'] : 1200;
        $textpanel_desc_length = (isset($params['textpanel_desc_length'])) ? (int) $params['textpanel_desc_length'] : 120;
        $isPagination = ($this->isPagination($enable_pagination, $theme)) ? true : false;
        $galleryItems = $this->getGalleryItems($gallery_id, $params, $isPagination, 0);
        $attachments = $galleryItems['items'];
        $total_items = $galleryItems['total_items'];
        if (empty($attachments)) {
            return '<h3>' . __('Gallery empty items!', 'wp-load-gallery') . '</h3>';
        }
        /* Create output html */
        if (!is_admin()) {
            $this->enqueue($theme);
        }

        $output = '';
        ob_start();
        $output .= '<div data-total_items="'. esc_attr($total_items) .'" class="wplg-wrap '. (((int) $border_box_animate === 1) ? 'wplg-border-gallery-box' : '') .' '. (($is_max_width) ? 'is_max_width' : '') .'" style="clear: both; margin: 10px 0; '. ((!$is_max_width) ? 'width: ' . (int) $gallery_width . 'px' : '') .'">';
        if (isset($theme)) {
            if (file_exists(WPLG_PLUGIN_DIR . 'frontend/themes/' . $theme . '/' . $theme . '.php')) {
                require(WPLG_PLUGIN_DIR . 'frontend/themes/' . $theme . '/' . $theme . '.php');
            } else {
                require(WPLG_PLUGIN_DIR . 'frontend/themes/default/default.php');
            }
        } else {
            require(WPLG_PLUGIN_DIR . 'frontend/themes/default/default.php');
        }
        $output .= ob_get_contents();
        ob_end_clean();
        $output .= $this->renderPagination($gallery_id, $is_max_width, $params, $total_items);
        $output .= '</div>';
        return $output;
    }

    public function loadmoreGallery()
    {
        if (empty($_REQUEST['wplg_nonce'])
            || !wp_verify_nonce($_REQUEST['wplg_nonce'], 'wplg_nonce')) {
            wp_send_json(
                array(
                    'status' => false
                )
            );
        }

        $params_request = json_decode(stripslashes($_REQUEST['options']), true);
        // sanitize options
        $options = Helper::sanitizeOptions($params_request);
        $params = $this->getSettingsFront($options['gallery_id'], array(), $options);
        $theme = get_term_meta((int) $options['gallery_id'], 'wplg_theme', true);
        $theme = Helper::getTheme($theme);
        foreach ($params as $attr_key => $attr_value) {
            ${$attr_key} = $attr_value;
        }

        $paged = (isset($_REQUEST['paged'])) ? (int) $_REQUEST['paged'] : 1;
        $offset = ($paged - 1) * (int) $items_per_page;
        $isPagination = ($this->isPagination($enable_pagination, $theme)) ? true : false;
        $galleryItems = $this->getGalleryItems($gallery_id, $params, $isPagination, $offset);
        $attachments = $galleryItems['items'];
        if (empty($attachments)) {
            wp_send_json(array('status' => false, 'paged' => $paged));
        }

        $theme = Helper::getTheme($theme);
        $load_more = 1;
        ob_start();
        if (isset($theme)) {
            if (file_exists(WPLG_PLUGIN_DIR . 'frontend/themes/' . $theme . '/' . $theme . '.php')) {
                require(WPLG_PLUGIN_DIR . 'frontend/themes/' . $theme . '/' . $theme . '.php');
            } else {
                require(WPLG_PLUGIN_DIR . 'frontend/themes/default/default.php');
            }
        } else {
            require(WPLG_PLUGIN_DIR . 'frontend/themes/default/default.php');
        }
        $output = ob_get_contents();
        ob_end_clean();
        wp_send_json(array('status' => true, 'items' => $output, 'paged' => $paged));
    }

    /**
     * @param $enable_pagination
     * @param $theme
     * @return bool
     */
    public function isPagination($enable_pagination, $theme)
    {
        return ((int) $enable_pagination === 1 && ($theme === 'flex' || $theme === 'square_grid' || $theme === 'default' || $theme === 'flip_box' || $theme === 'masonry' || $theme === 'portfolio'));
    }

    /**
     * Render load more
     *
     * @param integer $gallery_id
     * @param boolean $is_max_width
     * @param array   $params
     * @param integer $total_items
     *
     * @return string
     */
    public function renderPagination($gallery_id, $is_max_width, $params, $total_items = 0)
    {
        $pagination = '';
        if ($params['pagination_type'] === 'lazyload') {
            return $pagination;
        }
        $theme = get_term_meta((int) $gallery_id, 'wplg_theme', true);
        if ($this->isPagination($params['enable_pagination'], $theme)) {
            $total_pages = ceil((int)$total_items / (int)$params['items_per_page']);
            if ($is_max_width) {
                $pagination .= '<div class="wplg-pagnination-wrap" style="margin: 10px auto; width: 100%" data-gallery_id="' . esc_attr($gallery_id) . '" data-params="' . esc_attr(json_encode($params)) . '" data-total_pages="' . esc_attr($total_pages) . '">';
            } else {
                $pagination .= '<div class="wplg-pagnination-wrap" style="margin: 10px auto; width: ' . (int)$params['gallery_width'] . 'px" data-gallery_id="' . esc_attr($gallery_id) . '" data-params="' . esc_attr(json_encode($params)) . '" data-total_pages="' . esc_attr($total_pages) . '">';
            }

            if (isset($params['pagination_type']) && $params['pagination_type'] === 'number_page') {
                if ($total_pages > 1) {
                    $start = 1;
                    $end = 4;
                    if ($total_pages > 7) {
                        for ($i = $start; $i <= $end; $i++) {
                            if ((int)$i === 1) {
                                $pagination .= '<span class="wplg-number-page active-page" data-paged="' . esc_attr($i) . '">' . esc_html($i) . '</span>';
                            } else {
                                $pagination .= '<span class="wplg-number-page" data-paged="' . esc_attr($i) . '">' . esc_html($i) . '</span>';
                            }
                        }
                        $pagination .= '<span class="material-icons wplg-next-page" data-paged="' . esc_attr(((int)$end + 1)) . '"> navigate_next </span>';
                        $pagination .= '<span class="material-icons wplg-first-page" data-paged="' . esc_attr($total_pages) . '"> last_page </span>';
                    } else {
                        for ($i = 1; $i <= $total_pages; $i++) {
                            if ((int)$i === 1) {
                                $pagination .= '<span class="wplg-number-page active-page" data-paged="' . esc_attr($i) . '">' . esc_html($i) . '</span>';
                            } else {
                                $pagination .= '<span class="wplg-number-page" data-paged="' . esc_attr($i) . '">' . esc_html($i) . '</span>';
                            }
                        }
                    }
                }
            } else {
                $pagination .= '<button class="wplg_gallery_loadmore" data-gallery_id="' . esc_attr($gallery_id) . '" data-paged="2"><label>' . esc_html__('Load more', 'wp-load-gallery') . '</label><span class="material-icons-outlined wplg-loadmore-loader">sync</span></button>';
            }
            $pagination .= '</div>';
        }

        return $pagination;
    }

    /**
     * Get gallery items by gallery ID
     * @param integer $gallery_id Gallery ID
     * @param string  $wp_orderby Orderby
     * @param string  $wp_order   Order
     * @param string  $max_items  Max items
     *
     * @return array|int[]|WP_Post[]
     */
    public function getGalleryItems($gallery_id, $params = array(), $isPagination = false, $offset = 0)
    {
        $attachments = array();
        $ftp_sources = get_term_meta($gallery_id, 'wplg_gallery_server_folder', true);
        // get images from server folder
        if (!empty($ftp_sources) && is_array($ftp_sources)) {
            foreach ($ftp_sources as $ftp_source) {
                if (file_exists(ABSPATH . $ftp_source)) {
                    $ftp_files = glob(ABSPATH . $ftp_source . '/*.jpg');
                    foreach ($ftp_files as $ftp_file) {
                        if (is_file($ftp_file)) {
                            $types = wp_check_filetype($ftp_file);
                            if (strpos($types['type'], 'image/') !== false) {
                                $infos = pathinfo($ftp_file);
                                $xmp_list = wp_read_image_metadata($ftp_file);
                                $title = (isset($xmp_list['title']) && $xmp_list['title'] !== '') ? $xmp_list['title'] : $infos['filename'];
                                $desc = (isset($xmp_list['caption']) && $xmp_list['caption'] !== '') ? $xmp_list['caption'] : '';

                                $file = new stdClass();
                                $file->post_excerpt = $desc;
                                $file->post_title = $title;
                                $file->post_date = filemtime($ftp_file);
                                $file->url = str_replace(ABSPATH, site_url('/'), $ftp_file);
                                $file->path = $ftp_file;
                                $file->type = 'server_file';
                                $attachments[] = $file;
                            }
                        }
                    }
                }
            }
        }

        $items = Helper::getGalleryItems($gallery_id, $params['wp_orderby'], $params['wp_order']);
        foreach ($items as $item) {
            switch ($item->type) {
                case 'youtube':
                    $file = new stdClass();
                    $file->ID = $item->source_id;
                    $file->post_excerpt = $item->post_excerpt;
                    $file->post_title = $item->post_title;
                    $file->post_date = ($item->post_date !== '') ? strtotime($item->post_date) : time();
                    $file->url = $item->thumbnail;
                    $file->path = '';
                    $file->type = 'youtube';
                    $attachments[] = $file;
                    break;
                case 'vimeo':
                    $file = new stdClass();
                    $file->ID = $item->source_id;
                    $file->post_excerpt = $item->post_excerpt;
                    $file->post_title = $item->post_title;
                    $file->post_date = ($item->post_date !== '') ? strtotime($item->post_date) : time();
                    $file->url = $item->thumbnail;
                    $file->path = '';
                    $file->type = 'vimeo';
                    $attachments[] = $file;
                    break;
            }
        }

        // get item from post table
        $tax_query = array();
        $tax_query['relation'] = 'OR';
        $tax_query[] = array(
            'taxonomy' => WPLG_TAXONOMY,
            'field' => 'term_id',
            'terms' => $gallery_id,
            'include_children' => false
        );

        $categories = Helper::getGalleryCateogryItems($gallery_id);
        $find = false;
        if (!empty($categories)) {
            foreach ($categories as $category) {
                $sources = explode('***', $category->source_id);
                $category_id = (int) $sources[0];
                if (empty($sources[1])) {
                    continue;
                }
                $taxonomy = $sources[1];
                if ($category_id === 0) {
                    $tax_query[] = array(
                        'taxonomy' => $taxonomy,
                        'operator' => 'EXISTS'
                    );
                    $find = true;
                } else {
                    $tax_query[] = array(
                        'taxonomy' => $taxonomy,
                        'field' => 'term_id',
                        'terms' => $category_id,
                        'include_children' => ((int) $params['include_children'] === 1)
                    );
                }

            }
        }

        $max_items = (!empty($params['max_items'])) ? (int) $params['max_items'] : -1;
        /* Query images */
        $args = array(
            'posts_per_page' => $max_items,
            'post_status' => array('publish', 'inherit'),
            'post_type' => 'any',
            'order' => $params['wp_order'],
            'tax_query' => $tax_query,
            'meta_query' => array(
                'relation' => 'OR',
                array(
                    'key'     => 'wplg_image_status',
                    'compare' => 'NOT EXISTS'
                ),
                array(
                    'key'     => 'wplg_image_status',
                    'compare' => '=',
                    'value' => 1
                )
            )
        );

        // only user orderby if not random
        if ($params['wp_orderby'] !== 'rand' || !$isPagination) {
            $args['orderby'] = $params['wp_orderby'];
        }

        if ($find) {
            $args['posts_per_page'] = 50;
            $args['orderby'] = 'date';
            $args['order'] = 'DESC';
        }

        $query = new WP_Query($args);
        $medias_wordpress = $query->get_posts();

        // convert date of wordpress media to int
        if ((!empty($items) || !empty($ftp_sources)) && !empty($query->get_posts())) {
            if ($params['wp_orderby'] === 'date') {
                foreach ($medias_wordpress as &$medias_wordpres) {
                    $medias_wordpres->post_date = strtotime($medias_wordpres->post_date);
                }
            }
        }

        // merge all items
        $attachments = array_merge($attachments, $medias_wordpress);
        $total_items = count($attachments);

        // if order by is random and enable pagination, should get item by pagination before
        if ($params['wp_orderby'] === 'rand' && $isPagination) {
            $attachments = array_slice($attachments, $offset, $params['items_per_page']);
        }

        // sort again array item if include online source
        if (!empty($items) || !empty($ftp_sources) || ($isPagination && $params['wp_orderby'] === 'rand')) {
            switch ($params['wp_orderby']) {
                case 'title':
                    if ($params['wp_order'] === 'ASC') {
                        usort($attachments, function($a, $b) {return strcmp($a->post_title, $b->post_title);});
                    } else {
                        usort($attachments, function($a, $b) {return strcmp($b->post_title, $a->post_title);});
                    }
                    break;
                case 'date':
                    if ($params['wp_order'] === 'ASC') {
                        usort($attachments, function($a, $b) {return $a->post_date - $b->post_date;});
                    } else {
                        usort($attachments, function($a, $b) {return $b->post_date - $a->post_date;});
                    }
                    break;
                case 'rand':
                    shuffle($attachments);
                    break;
            }
        }

        // if order by isn't random and enable pagination, should get item by pagination after
        if ($params['wp_orderby'] !== 'rand' && $isPagination) {
            $attachments = array_slice($attachments, $offset, $params['items_per_page']);
        }
        return array('items' => $attachments, 'total_items' => $total_items);
    }

    /**
     * Load scripts and styles
     *
     * @param string  $theme              Theme name
     *
     * @return void
     */
    public function enqueue($theme)
    {
        wp_enqueue_script('jquery');
        wp_enqueue_style(
            'wp-material-icon',
            'https://fonts.googleapis.com/css?family=Material+Icons|Material+Icons+Outlined'
        );
        wp_enqueue_script(
            'wplg-magnific-popup-script',
            WPLG_PLUGIN_URL . '/assets/js/magnific-popup/jquery.magnific-popup.min.js',
            array('jquery'),
            '1.1.0',
            true
        );

        wp_enqueue_style(
            'wplg-magnific-popup-style',
            WPLG_PLUGIN_URL . '/assets/js/magnific-popup/magnific-popup.css',
            array(),
            '1.1.0'
        );

        wp_enqueue_script(
            'wplg-imagesloaded',
            WPLG_PLUGIN_URL . 'assets/js/imagesloaded.min.js',
            array(),
            '3.1.5',
            true
        );

        if ($theme === 'slider') {
            wp_enqueue_style(
                'wp-slick-style',
                WPLG_PLUGIN_URL . 'assets/js/slick/slick.css',
                array(),
                WPLG_VERSION
            );

            wp_enqueue_style(
                'wp-slick-theme-style',
                WPLG_PLUGIN_URL . 'assets/js/slick/slick-theme.css',
                array(),
                WPLG_VERSION
            );

            wp_enqueue_script(
                'wp-slick-script',
                WPLG_PLUGIN_URL . 'assets/js/slick/slick.min.js',
                array('jquery'),
                WPLG_VERSION
            );
        }

        wp_enqueue_style(
            'wp-gallery-css',
            WPLG_PLUGIN_URL . '/assets/css/gallery.css',
            array(),
            WPLG_VERSION
        );

        wp_enqueue_script(
            'wp-gallery-js',
            WPLG_PLUGIN_URL . '/assets/js/gallery.js',
            array('jquery'),
            WPLG_VERSION,
            true
        );
        wp_localize_script('wp-gallery-js', 'wpgallery', $this->localizeScript());
    }

    /**
     * Localize a script.
     * Works only if the script has already been added.
     *
     * @return array
     */
    public function localizeScript()
    {
        return array(
            'wplg_nonce' => wp_create_nonce('wplg_nonce'),
            'ajaxurl' => admin_url('admin-ajax.php')
        );
    }

    /**
     * Get settings front
     *
     * @param integer $gallery_id Gallery ID
     * @param array $request_settings Request settings
     * @param array $options Gallery configs
     *
     * @return array
     */
    public function getSettingsFront($gallery_id, $request_settings, $options)
    {
        if (empty($options)) {
            $options = array();
        }
        $default = Helper::getDefaultOptions();
        $default['gallery_id'] = (int) $gallery_id;
        $settings = array_merge($default, $options, $request_settings);
        $theme = get_term_meta((int) $gallery_id, 'wplg_theme', true);
        $settings['theme'] = $theme;
        if ((int) $settings['border_box_animate'] === 1) {
            $settings['textpanel_bg_color'] = 'transparent';
        }
        return $settings;
    }
}
